package main

import (
  "log"
  "github.com/gofiber/fiber/v2"
  "github.com/gofiber/fiber/v2/middleware/logger"

   _ "github.com/mattn/go-sqlite3"
   "github.com/jmoiron/sqlx"
   sq "github.com/Masterminds/squirrel"

   "github.com/go-playground/validator/v10"
)

type Link struct {
  Id int `db:"id" json:"id"`
  StreamId int `db:"stream_id" json:"stream_id" form:"stream_id" validate:"required,numeric"`
  Url string `db:"url" json:"url" form:"url" validate:"required,url"`
  Description string `db:"description" json:"description" form:"description" validate:"required"`
}

type Stream struct {
  Id int `db:"id" json:"id"`
  Title string `db:"title" json:"title"`
  Description string `db:"description" json:"description"`
}


func SelectJson[T any](db *sqlx.DB, c *fiber.Ctx, err error, sql string, args ...interface{}) error {
  if(err != nil) {
    log.Fatalln(err)
  }
  var result []T

  err = db.Select(&result, sql, args...)
  if(err != nil) {
    log.Fatalln(err);
  }

  return c.JSON(&result);
}

func GetJson[T any](db *sqlx.DB, c *fiber.Ctx, err error, sql string, args ...interface{}) error {
  if(err != nil) {
    log.Fatalln(err)
  }
  var result T

  err = db.Get(&result, sql, args...)
  if(err != nil) {
    log.Fatalln(err);
  }

  return c.JSON(&result);
}

func main() {
  log.SetFlags(log.LstdFlags | log.Lshortfile)

  db, err := sqlx.Connect("sqlite3", "db.sqlite3")
  if err != nil {
      log.Fatalln(err)
  }

  app := fiber.New()
  app.Use(logger.New())

  // handler that returns one json from a sql database
  app.Get("/api/stream/", func (c *fiber.Ctx) error {
    sql, args, err := sq.Select("*").From("stream").ToSql()
    return SelectJson[Stream](db, c, err, sql, args...)
  })

  app.Get("/api/stream/:id", func (c *fiber.Ctx) error {
    sql, args, err := sq.Select("*").From("stream").Where("id", c.Params("id")).ToSql()
    return GetJson[Stream](db, c, err, sql, args...)
  })

  app.Get("/api/stream/:id/links", func (c *fiber.Ctx) error {
    sql, args, err := sq.Select("*").From("stream_link").Where("stream_id", c.Params("id")).ToSql()

    return SelectJson[Link](db, c, err, sql, args...)
  })

  app.Post("/api/link", func (c *fiber.Ctx) error {
    link := new(Link)

    if err := c.BodyParser(link); err != nil {
      log.Println(err);
      return c.Redirect("/live/")
    }

    var validate *validator.Validate
    validate = validator.New(validator.WithRequiredStructEnabled())

    if err := validate.Struct(link); err != nil {
      validationErrors := err.(validator.ValidationErrors)
      log.Println(validationErrors)
      return c.Redirect("/live/")
    }

    sql, args, err := sq.Insert("stream_link").Columns("stream_id", "url", "description").Values(link.StreamId, link.Url, link.Description).ToSql()

    if err != nil {
      log.Println(err)
      return c.Redirect("/live/")
    }

    db.MustExec(sql, args...)

    return c.Redirect("/live/")
  })

  app.Static("/", "./public")

  log.Fatal(app.Listen(":5001"))
}
